#!/usr/bin/env python

# Examples demonstrating the use of the Wumpus package

import argparse
import random
import sys

import wumpus as wws


class MyPlayer(wws.OfflinePlayer, wws.UserPlayer):
    """Offline player demonstrating the use of the start episode method to inspect the world."""

    def start_episode(self, world: wws.WumpusWorld):
        """Print the description of the world before starting."""

        world_info = {k: [] for k in ('Hunter', 'Pits', 'Wumpus', 'Gold', 'Exits')}
        world_info['Size'] = (world.size.x, world.size.y)
        world_info['Blocks'] = [(c.x, c.y) for c in world.blocks]

        for obj in world.objects:
            if isinstance(obj, wws.Hunter):
                world_info['Hunter'].append((obj.location.x, obj.location.y))
            elif isinstance(obj, wws.Pit):
                world_info['Pits'].append((obj.location.x, obj.location.y))
            elif isinstance(obj, wws.Wumpus):
                world_info['Wumpus'].append((obj.location.x, obj.location.y))
            elif isinstance(obj, wws.Exit):
                world_info['Exits'].append((obj.location.x, obj.location.y))
            elif isinstance(obj, wws.Gold):
                world_info['Gold'].append((obj.location.x, obj.location.y))

        print('World details:')
        for k in ('Size', 'Pits', 'Wumpus', 'Gold', 'Exits', 'Blocks'):
            print('  {}: {}'.format(k, world_info.get(k, None)))


def classic(size: int = 0):
    """Play the classic version of the wumpus."""
    # create the world
    world = wws.WumpusWorld.classic(size=size if size > 3 else random.randint(4, 8))

    # Run a player without any knowledge about the world
    wws.run_episode(world, wws.UserPlayer())


def classic_offline(size: int = 0):
    """Play the classic version of the wumpus with a player knowing the world and the agent."""
    # create the world
    world = wws.WumpusWorld.classic(size=size if size > 3 else random.randint(4, 8))

    # Run a player with knowledge about the world
    wws.run_episode(world, MyPlayer())


WUMPUS_WORLD = '''
    {
        "id": "simple wumpus world",
        "size": [7, 7],
        "hunters": [[0, 0]],
        "pits": [[4, 0], [3, 1], [2, 2], [6, 2], [4, 4], [3, 5], [4, 6], [5, 6]],
        "wumpuses": [[1, 2]],
        "exits": [[0, 0]],
        "golds": [[6, 3], [3, 3]],
        "blocks": []
    }
'''


def fixed_offline(world_json: str = WUMPUS_WORLD):
    """Play on a given world described in JSON format."""
    # create the world
    world = wws.WumpusWorld.from_JSON(world_json)

    # Run a player with knowledge about the world
    wws.run_episode(world, MyPlayer())


def real_deal(size: int = 0):
    """Play the classic version of the wumpus without being able to see the actual layout, that is as the actual software agent will do."""
    # create the world
    world = wws.WumpusWorld.classic(size=size if size > 3 else random.randint(4, 8))

    # Run a player without any knowledge about the world
    wws.run_episode(world, wws.UserPlayer(), show=False)


EXAMPLES = (classic, classic_offline, fixed_offline, real_deal)


def main(*cargs):
    """Demonstrate the use of the wumpus API on selected worlds"""
    ex_names = {ex.__name__.lower(): ex for ex in EXAMPLES}
    parser = argparse.ArgumentParser(description=main.__doc__, formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('example', nargs='?', help='select one of the available example', choices=list(ex_names.keys()))
    args = parser.parse_args(cargs)
    
    if args.example:
        ex = ex_names[args.example.lower()]
    else:
        # Randomly play one of the examples
        ex = random.choice(EXAMPLES)

    print('Example {}:'.format(ex.__name__))
    print('  ' + ex.__doc__)
    ex()

    return 0


if __name__ == "__main__":
    sys.exit(main(*sys.argv[1:]))
